package IniConf::Tied;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use Carp;
use IniConf;

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
	
);
$VERSION = '0.01';

######################################################################
# Standard tied hash functions
sub TIEHASH 
{
    my($package, $file)=@_;
    my $ini=IniConf->new('-file'=>$file);
    return unless $ini;
    my $self=bless {ini=>$ini, sections=>{}}, $package;
    @{$self->{sections}}{$ini->Sections()}=();
    return $self;
}

sub STORE
{
    my($self, $key, $value)=@_;
    carp "You may not create new sections in an INI file.";
}

sub FETCH
{
    my($self, $key)=@_;
    return {} unless exists $self->{sections}{$key};
    my %section;
    tie %section, __PACKAGE__.'::Section', $self->{ini}, $key;
    return \%section;
}

sub FIRSTKEY
{
    my($self)=@_;
    $self->{temp}=[keys %{$self->{sections}}];
    return unless @{$self->{temp}};
    return pop @{$self->{temp}};
}

sub NEXTKEY
{
    my($self, $lastkey)=@_;
    $self->{temp}||=[keys %{$self->{sections}}];
    return unless @{$self->{temp}};
    return pop @{$self->{temp}};
}

sub EXISTS
{
    my($self, $key)=@_;
    return exists $self->{sections}->{$key};
}

sub DELETE
{
    my($self, $key)=@_;
    $self->{temp}=[];
    delete $self->{'sections'}{$key};
}

sub CLEAR
{
    my($self)=@_;
    $self->{temp}=[];
    $self->{'sections'}={};
    return;
}

######################################################################
package IniConf::Tied::Section;

use strict;
use Carp;
use Data::Dumper;

sub TIEHASH 
{
    my($package, $ini, $section)=@_;
    my $self=bless {ini=>$ini, section=>$section}, $package;
    return $self;
}

sub STORE
{
    my($self, $key, $value)=@_;
    return unless $key;
    unless(defined $self->{ini}->setval($self->{section}, $key, $value))
    {
        carp "Attempting to set '$key', not defined in [$self->{section}]";
    }
}

sub FETCH
{
    my($self, $key)=@_;
    return $self->{ini}->val($self->{section}, $key);
}

sub FIRSTKEY
{
    my($self)=@_;
    $self->{temp}=[$self->{ini}->Parameters($self->{section})];
    return unless @{$self->{temp}};
    return pop @{$self->{temp}};
}

sub NEXTKEY
{
    my($self, $lastkey)=@_;
    $self->{temp}||=[$self->{ini}->Parameters($self->{section})];
    return unless @{$self->{temp}};
    return pop @{$self->{temp}};
}

sub EXISTS
{
    my($self, $key)=@_;
    return defined $self->{ini}->val($self->{section}, $key) 
}

sub DELETE
{
    my($self, $key)=@_;
    return unless $key;
    unless(defined $self->{ini}->setval($self->{section}, $key, undef()))
    {
        carp "Attempting to delete '$key', not defined in [$self->{section}]";
    }
}

sub CLEAR
{
    my($self)=@_;
    $self->{temp}=[];
    return;
}


1;
__END__

=head1 NAME

IniConf::Tied - Tied hash interface to IniConf

=head1 SYNOPSIS

  use IniConf::Tied;
  my %foo;
  tie %foo, 'IniConf::Tied', '_test.ini' or die "Unable to read _test.ini";

  my $value=$foo{'section_name'}->{'param_name'};

  foreach my $param (sort keys %{$foo{'section1'}})
  {
    print "$param is $foo{section1}{$param}\n";
  }

  # a better way to do the previous 
  my $section1=$foo{'section1'};
  foreach my $p (sort keys %$section1)
  {
    print "$p is $section1->{$p}\n";
  }

=head1 DESCRIPTION

IniConf::Tied is a tied hash wraper for IniConf.  It allows you to treat an 
INI file as a read-only hash.   This allows you to use an INI file in places
where you have to use a hash, rather then copying a section to a temporary
hash.

IniConf::Tied works it's magic by creating a reference to a temporary tied
hash when you ask for a section (ie $ini{section_name}).  You then pull
values out of that hashref when you access parameters (ie
$ini{section_name}{parameter_name}).  This comes at the expense of speed,
however.  Tied hashes FETCHes are about 3 times slower then a method
call.  Also, doing $ini{s}{p} creates a temporary object which is promptly
thrown away.  So, if you are accessing a give section often, you should keep
a reference to the intermediat value.  This is done in the second foreach
loop in L</SYNOPSIS>.

Because INI files are basicaly read only, IniConf::Tied is read only for the
most part.  You may delete or modify existing parameters.  You may also
delete existing sections.  However, you may not modify existing sections nor
create new sections or parameters.  Modifying the IniConf::Tied data will not
change the INI file on disk, however.

Give the file foo.ini :

    [bar]
    hello=world

    [biff]
    boof=silly

    delete $foo{bar};       # section bar is no longer available
    delete $foo{biff}{boof} # parameter boof in section biff is now gone
    $foo{biff}{boof}=1;     # this works because the parameter already existed
    
    # The following doesn't work
    $foo{baz}=\%hash;       # NO creating a new section
    $foo{bar}=\%hash;       # NO modifying a section
    $foo{bar}{billy}='bob'; # NO creating an new parameter

=head1 AUTHOR

Philip Gwyn <perl@pied.nu>

=head1 SEE ALSO

L<IniConf>

=cut
